import gradio as gr
import openai
import redis

Title = "Experience the Power of Knowledge with Smart QnA Bot with Redis"
Description = "The smart QnA bot is a tool that can help you find answers to your questions quickly and easily. Using advanced algorithms and a vast knowledge base, the bot can provide comprehensive responses to a wide range of inquiries. However, the accuracy of the answers may vary, and they should always be cross-checked with other sources to ensure their reliability. Whether you're a student, a professional, or just someone seeking information, the QnA bot is a useful resource for exploring different topics and expanding your knowledge. Give it a try today and experience the convenience and efficiency of this smart tool."

# Connect to Redis
redis_client = redis.Redis(host='redis', port=6379, db=0)

def openai_call(question, api_key):
    # Check if the answer is already in Redis cache
    cached_answer = redis_client.get(question)
    if cached_answer:
        return cached_answer.decode('utf-8')

    # If answer is not in cache, call OpenAI API
    openai.api_key = api_key
    response = openai.Completion.create(
        model="text-davinci-003",
        prompt=question,
        temperature=0,
        max_tokens=100
    )
    answer = response.choices[0].text

    # Store the answer in Redis cache for future requests
    redis_client.set(question, answer)

    return answer

open_ai_key = gr.inputs.Textbox(label="Enter the Open AI API Key",type="password")
open_ai_question = gr.inputs.Textbox(lines=10, label="Enter the question")  
open_ai_answer = gr.outputs.Label(label="Answer")
demo = gr.Interface(title=Title, description=Description, fn=openai_call,inputs=[open_ai_question,open_ai_key], outputs=[open_ai_answer],css="footer {visibility: hidden}")

demo.launch(server_name="0.0.0.0", server_port=8081)
